/*

Simple Joystick Demo by Madman
Feel free to use and modify as needed

*/

import "mod_key"
import "mod_joy"
import "mod_proc"

CONST
	// This defines readable names for indexing the array.
 	UP = 1;
    DOWN = 2;
    LEFT = 3;
    RIGHT = 4;
    
    BA = 5;
    BB = 6;
    BX = 7;
    BY = 8;
    
    START = 9;
    SELECT = 10;
    
	VOL_UP = 11;
	VOL_DOWN = 12;

	LS = 13;
	RS = 14;

	ESC = 15;
	FIRE = 16; // Special Fake Key for 2 Player Modes. 
	
	UP_LEFT = 17;
	UP_RIGHT = 18;
	DOWN_LEFT = 19;
	DOWN_RIGHT = 20;
	
    NUM_OF_KEYS = 20;
END

GLOBAL
int KEY[NUM_OF_KEYS];
int JOY[NUM_OF_KEYS];
int JOY2[NUM_OF_KEYS];
END


FUNCTION init_controls();
// Set default keys and run control process
BEGIN
	set_default_keys(); // set default keys :)
	signal( type wiz_controls, s_kill ); // Kill any running processes - We only need this once
	wiz_controls(); // Start process to retrieve controls
END



FUNCTION set_default_keys()
// Define default keys to be used on Windows.
// This way we can test the programm on Windows
BEGIN
    KEY[UP]=_w;
    KEY[DOWN]=_s;
    KEY[LEFT]=_a;
    KEY[RIGHT]=_d;
    
    KEY[BA]=_1;
    KEY[BB]=_2;
    KEY[BX]=_3;
    KEY[BY]=_4;
    
    KEY[START]=_enter;
    KEY[SELECT]=_m;
    
    KEY[LS]=_q;
    KEY[RS]=_e;

	KEY[VOL_UP]=_up;
 	KEY[VOL_DOWN]=_down;
 	
 	KEY[ESC]=_esc;
 	KEY[FIRE]=_space;
 	    

END


PROCESS wiz_controls()
private
  k;
BEGIN
	
    #define WIZ_UP          0
    #define WIZ_UP_LEFT     1
    #define WIZ_LEFT        2
    #define WIZ_DOWN_LEFT   3
    #define WIZ_DOWN        4
    #define WIZ_DOWN_RIGHT  5
    #define WIZ_RIGHT       6
    #define WIZ_UP_RIGHT    7

    #define WIZ_MENU        8
    #define WIZ_SELECT      9

    #define WIZ_L_BUTTON    10
    #define WIZ_R_BUTTON    11

    #define WIZ_A           12
    #define WIZ_B           13
    #define WIZ_X           14
    #define WIZ_Y           15

    #define WIZ_VOL_UP      16
    #define WIZ_VOL_DOWN    17
   
    loop
 		// Get keys pressed and assign to JOY
        for ( k = 1; k <= NUM_OF_KEYS; k++)
                JOY[k] = key(KEY[k]);
        end

        // Check Wiz buttons 

        if (joy_number())
            JOY[UP]    |= joy_getbutton(0,WIZ_UP);
            JOY[DOWN]  |= joy_getbutton(0,WIZ_DOWN);
            JOY[LEFT]  |= joy_getbutton(0,WIZ_LEFT);
            JOY[RIGHT] |= joy_getbutton(0,WIZ_RIGHT);
            
            JOY[UP_LEFT]    |= joy_getbutton(0,WIZ_UP_LEFT);
            JOY[UP_RIGHT]  |= joy_getbutton(0,WIZ_UP_RIGHT);
            JOY[DOWN_LEFT]  |= joy_getbutton(0,WIZ_DOWN_LEFT);
            JOY[DOWN_RIGHT] |= joy_getbutton(0,WIZ_DOWN_RIGHT);
            
            JOY[BA]    |= joy_getbutton(0,WIZ_A);
            
            JOY[BB]    |= joy_getbutton(0,WIZ_B);
            JOY[BX]    |= joy_getbutton(0,WIZ_X);
            JOY[BY]    |= joy_getbutton(0,WIZ_Y);
            
            JOY[START] |= joy_getbutton(0,WIZ_MENU);
            JOY[SELECT] |= joy_getbutton(0,WIZ_SELECT);
            
            JOY[VOL_UP] |= joy_getbutton(0,WIZ_VOL_UP);
            JOY[VOL_DOWN] |= joy_getbutton(0,WIZ_VOL_DOWN);
			
			JOY[LS] |= joy_getbutton(0,WIZ_L_BUTTON);           
			JOY[RS] |= joy_getbutton(0,WIZ_R_BUTTON);   
			
			if (JOY[LS] && JOY[RS] && JOY[START])
				JOY[ESC] = 1;
			end
			
			/*
			
			We may define some alias here. In case you like to do a 2 Player game,
			you can check for JOY2[UP] instead of JOY[Y] which helps keep the code readable.
			If you need this, you can uncomment it.
			
			JOY[FIRE] = JOY[START];
			JOY2[FIRE] = JOY[SELECT];
			JOY2[LEFT] = JOY[BA];
			JOY2[RIGHT] = JOY[BB];
			JOY2[UP] = JOY[BY];
			JOY2[DOWN] = JOY[BX];
			
			*/
          
        END
       
        frame;
    END
END